'use strict';

var _state = require('./state');

/**
 * Copyright (c) 2014-present, Facebook, Inc. All rights reserved.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 *  strict-local
 */

const describe = (blockName, blockFn) => _dispatchDescribe(blockFn, blockName);
describe.only = (blockName, blockFn) =>
  _dispatchDescribe(blockFn, blockName, 'only');
describe.skip = (blockName, blockFn) =>
  _dispatchDescribe(blockFn, blockName, 'skip');

const _dispatchDescribe = (blockFn, blockName, mode) => {
  (0, _state.dispatch)({
    asyncError: new Error(),
    blockName,
    mode,
    name: 'start_describe_definition'
  });
  blockFn();
  (0, _state.dispatch)({blockName, mode, name: 'finish_describe_definition'});
};

const _addHook = (fn, hookType, timeout) =>
  (0, _state.dispatch)({
    asyncError: new Error(),
    fn,
    hookType,
    name: 'add_hook',
    timeout
  });
const beforeEach = (fn, timeout) => _addHook(fn, 'beforeEach', timeout);
const beforeAll = (fn, timeout) => _addHook(fn, 'beforeAll', timeout);
const afterEach = (fn, timeout) => _addHook(fn, 'afterEach', timeout);
const afterAll = (fn, timeout) => _addHook(fn, 'afterAll', timeout);

const test = (testName, fn, timeout) => {
  if (typeof testName !== 'string') {
    throw new Error(
      `Invalid first argument, ${testName}. It must be a string.`
    );
  }
  if (fn === undefined) {
    throw new Error('Missing second argument. It must be a callback function.');
  }
  if (typeof fn !== 'function') {
    throw new Error(
      `Invalid second argument, ${fn}. It must be a callback function.`
    );
  }

  const asyncError = new Error();
  if (Error.captureStackTrace) {
    Error.captureStackTrace(asyncError, test);
  }

  return (0, _state.dispatch)({
    asyncError,
    fn,
    name: 'add_test',
    testName,
    timeout
  });
};
const it = test;
test.skip = (testName, fn, timeout) => {
  const asyncError = new Error();
  if (Error.captureStackTrace) {
    Error.captureStackTrace(asyncError, test);
  }

  return (0, _state.dispatch)({
    asyncError,
    fn,
    mode: 'skip',
    name: 'add_test',
    testName,
    timeout
  });
};
test.only = (testName, fn, timeout) => {
  const asyncError = new Error();
  if (Error.captureStackTrace) {
    Error.captureStackTrace(asyncError, test);
  }

  return (0, _state.dispatch)({
    asyncError,
    fn,
    mode: 'only',
    name: 'add_test',
    testName,
    timeout
  });
};

module.exports = {
  afterAll,
  afterEach,
  beforeAll,
  beforeEach,
  describe,
  it,
  test
};
